    /*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package channel;

import java.io.Serializable;
import math.Polynomial;
import parser.Constants;

/**
 * Class representing a channel with raw and calibrated data from the car sensors.
 * @author Lauritz Møllersen  
 */
public class Channel implements Serializable {

    private String name = "";
    private Unit unit; // not currently in use
    private double[] values;
    private double[] calibratedValues = values;
    private Polynomial polynomials = null;
    
    // -1 = math/created, 1 = distance, 2 = time, 3 = speed, 4 = throttle, 5 = brake, 6 = latacc, 7 = lap, gps pos x = 8, gps pos y = 9
    private int type = Constants.CHAN_OTHER;

    @Override
    public String toString(){
        return name;
    }
    
    /**
     * Constructor with name.
     * @param tempName The channel's name.
     */
    public Channel(String tempName) {
        name = tempName;
    }
    
    /**
     * 
     * @param tempName
     * @param type
     */
    public Channel(String tempName, int type) {
        name = tempName;
        this.type = type;
    }

    /**
     * Constructor with name and channel data.
     * @param tempValues The channel's data.
     * @param tempName The channel's name.
     */
    public Channel(double[] tempValues, String tempName) {
        values = tempValues;
        name = tempName;
        calibratedValues = tempValues;
    }

    /**
     * @return the name
     */
    public String getName() {
        return name;
    }

    /**
     * @param name the name to set
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * @return the values
     */
    public double[] getValues() {
        return values;
    }

    /**
     * @param values the values to set
     */
    public void setValues(double[] values) {
        this.values = values;
        calibratedValues = values;
    }

    /**
     * @return the unit
     */
    public Unit getUnit() {
        return unit;
    }

    /**
     * @param unit the unit to set
     */
    public void setUnit(Unit unit) {
        this.unit = unit;
    }

    /**
     * @return the type
     */
    public int getType() {
        return type;
    }

    /**
     * @param type the type to set
     */
    public void setType(int type) {
        this.type = type;
    }

    /**
     * @return the calibratedValues
     */
    public double[] getCalibratedValues() {
        return calibratedValues;
    }

    /**
     * @param calibratedValues the calibratedValues to set
     */
    public void setCalibratedValues(double[] calibratedValues) {
        this.calibratedValues = calibratedValues;
    }

    /**
     * @return the polynomials for use in the Calibration process
     */
    public Polynomial getPolynomials() {
        return polynomials;
    }

    /**
     * @param polynomials the polynomials to set
     */
    public void setPolynomials(Polynomial polynomials) {
        this.polynomials = polynomials;
    }
}
