/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package gui;

import java.awt.Rectangle;
import java.util.ArrayList;
import project.Project;
import plugin.Plugin;
import view.View;

/**
 * The main GUI interface.
 * @author Per Øyvind Stadheim
 */
public interface MainGUI {

    /**
     * Sets the title of the main GUI frame.
     * @param title
     */
    public void setFrameTitle(String title);

    /**
     * Sets the project reference in the MainGUI object.
     * @param p The project object.
     */
    public void setProject(Project p);

    /**
     * Gets the project reference from a project object. 
     * @return Project reference.
     */
    /**
     * Adds a view module to the user interface.
     * @param v The view to add.
     * @param pos  
     */
    public void addView(View v, int pos);

    /**
     * Removes a view module from the user interface.
     * @param v The view to remove.
     */
    public void removeView(View v);

    /**
     * Returns the view component having focus at the current time.
     * @return The focused view.
     */
    public View getCurrentView();

    /**
     * Returns the plugin component having focus at the current time.
     * @return The focused plugin.
     */
    public Plugin getCurrentPlugin();

    /**
     * Returns the dimension of the view area. This is used
     * by the view module when initializing its GUI component.
     * @return Dimension
     */
    public Rectangle getViewAreaDimension();

    /**
     * Sets the plugin to be displayed in the plugin properties panel.
     * @param p The plugin to be displayed.
     */
    public void setCurrentPlugin(Plugin p);

    /**
     * Enables/disables the undo button
     * @param b Value
     */
    public void setUndoEnabled(boolean b);

    /**
     * Enables/disables the redo button
     * @param b
     */
    public void setRedoEnabled(boolean b);

    /**
     * Sets the undo caption text
     * @param text Caption text
     */
    public void setUndoText(String text);

    /**
     * Sets the redo caption text
     * @param text Caption text
     */
    public void setRedoText(String text);

    /**
     * Adds external plugins to the Plugin menu
     * @param classes External Plugins
     */
    public void addExternalPluginEntries(ArrayList<Class> classes);

    /**
     * Method used to update the titles of all views.
     */
    public void viewTitlesChanged();
}