/***************************************************************************
 *   Copyright (C) 2009 by Paul Lutus, Ian Clarke                          *
 *   lutusp@arachnoid.com, ian.clarke@gmail.com                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *

 * A class to fit a polynomial to a (potentially very large) dataset.
 * 
 * @author Paul Lutus <lutusp@arachnoid.com>
 * @author Ian Clarke <ian.clarke@gmail.com>
 * 

 * Changelog:
 * 20100130: Add note about relicensing
 * 20091114: Modify so that points can be added after the curve is
 *           created, also some other minor fixes 
 * 20091113: Extensively modified by Ian Clarke, main changes:
 *     - Should now be able to handle extremely large datasets
 *     - Use generic Java collections classes and interfaces 
 *       where possible
 *     - Data can be fed to the fitter as it is available, rather
 *       than all at once
 * 
 * The code that this is based on was obtained from: http://arachnoid.com/polysolve
 * 
 * Note: I (Ian Clarke) am happy to release this code under a more liberal 
 * license such as the LGPL, however Paul Lutus (the primary author) refuses
 * to do this on the grounds that the LGPL is not an open source license.  
 * If you want to try to explain to him that the LGPL is indeed an open 
 * source license, good luck - its like talking to a brick wall.
 */

package math;

import java.io.Serializable;
import java.util.ArrayList;

/**
 * 
 * @author Per Øyvind
 */
public class Polynomial extends ArrayList<Float> implements Serializable{

    /**
     * Default constructor.
     * @param p Amount of polynomials.
     */
    public Polynomial(final int p) {
        super(p);
    }
    
    /**
     * Default constructor.
     */
    public Polynomial() {
        super();
    }

    /**
     * Method for caluclating the y-value based on the current polynommials and an x-value.
     * @param x The x-value.
     * @return The y-value.
     */
    public double getY(final double x) {
        double ret = 0;
        for (int p = 0; p < size(); p++) {
            ret += get(p) * (Math.pow(x, p));
        }
        return ret;
    }

    @Override
    public String toString() {
        final StringBuilder ret = new StringBuilder();
        for (int x = size() - 1; x > -1; x--) {
            ret.append(get(x) + (x > 0 ? "x^" + x + " + " : ""));
        }
        return ret.toString();
    }
}
