/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package plugin;

import java.io.File;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.URLClassLoader;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author Per Øyvind
 */
public class PluginDependencyLoader {

    /**
     * Adds a file to the classpath.
     * @param f
     */
    public static void addFile(File f) {
        URL[] url = new URL[1];
        try {
            url[0] = f.toURI().toURL();
        } catch (MalformedURLException ex) {
            ex.printStackTrace();
        }
        PluginDependencyLoader.addURLs(url);
    }

    /**
     * Adds a list of URLs to the classpath.
     * @param urls
     */
    public static void addURLs(URL[] urls) {
        ClassLoader cl = ClassLoader.getSystemClassLoader();
        if (cl instanceof URLClassLoader) {
            URLClassLoader ul = (URLClassLoader) cl;
            // addURL is a protected method, but we can use reflection to call it
            Class<?>[] paraTypes = new Class[1];
            paraTypes[0] = URL.class;
            Method method;
            try {
                method = URLClassLoader.class.getDeclaredMethod("addURL", paraTypes);
                method.setAccessible(true);
                Object[] args = new Object[1];
                for (int i = 0; i < urls.length; i++) {
                    args[0] = urls[i];
                    try {
                        method.invoke(ul, args);
                    } catch (IllegalAccessException ex) {
                        Logger.getLogger(PluginFactory.class.getName()).log(Level.SEVERE, null, ex);
                    } catch (IllegalArgumentException ex) {
                        Logger.getLogger(PluginFactory.class.getName()).log(Level.SEVERE, null, ex);
                    } catch (InvocationTargetException ex) {
                        Logger.getLogger(PluginFactory.class.getName()).log(Level.SEVERE, null, ex);
                    }
                }
            } catch (NoSuchMethodException ex) {
                Logger.getLogger(PluginFactory.class.getName()).log(Level.SEVERE, null, ex);
            } catch (SecurityException ex) {
                Logger.getLogger(PluginFactory.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    /**
     * Returns the jar files found in the working directory.
     * @return List of files
     */
    public static ArrayList<File> getJarFiles() {
        File dir = null;
        try {
            URI uri = PluginFactory.class.getProtectionDomain().getCodeSource().getLocation().toURI();
            String parts = uri.getPath().substring(1, uri.getPath().lastIndexOf("/") + 1);
            String result;
            try {
                result = java.net.URLEncoder.encode(parts, "UTF-8");
                dir = new File(URLDecoder.decode(result, "UTF-8"));
            } catch (UnsupportedEncodingException ex) {
                Logger.getLogger(PluginFactory.class.getName()).log(Level.SEVERE, null, ex);
            }
        } catch (URISyntaxException ex) {
            Logger.getLogger(PluginFactory.class.getName()).log(Level.SEVERE, null, ex);
        }
        ArrayList<File> fileList = new ArrayList<File>();
        String[] children = dir.list();
        if (children != null) {
            for (int i = 0; i < children.length; i++) {
                String filename;
                filename = children[i];
                try {
                    File f = new File(URLDecoder.decode(dir.getAbsolutePath() + "/" + filename, "UTF-8"));

                    if (f.canRead() && f.getName().contains(".jar")) {
                        fileList.add(f);
                    }
                } catch (UnsupportedEncodingException ex) {
                    Logger.getLogger(PluginFactory.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }

        return fileList;
    }

    /**
     * Adds any jar files in the working directory into the classpath
     */
    public static void loadNearByDependencies() {
        ArrayList<File> files = PluginDependencyLoader.getJarFiles();
        for (File f : files) {
            PluginDependencyLoader.addFile(f);
        }
    }
}
