/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package plugin.report;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.table.TableCellRenderer;

/**
 * This class represents a report JTable for use in the GUI.
 * @author Lauritz Møllersen
 */
public class ReportTable extends JTable {

    private boolean DEBUG = false;
    private int[] green;
    private final int size;

    /**
     * Default constructor.
     * @param headers The header of the table.
     * @param data The data of the table.
     */
    public ReportTable(String[] headers, Object[][] data) {
        super(data, headers);

        size = data[0].length;
        green = findBestSegments(data);

        setPreferredScrollableViewportSize(new Dimension(500, 70));
        setFillsViewportHeight(true);
        setOpaque(true); //content panes must be opaque
        getColumnModel().getColumn(0).setMinWidth(95);
        setDragEnabled(false);
        getTableHeader().setReorderingAllowed(false);
    }

    @Override
    public boolean isCellEditable(int rowIndex, int vColIndex) {
        return false;
    }

    // overriding the preParerenderer to change the background color of the cells.
    @Override
    public Component prepareRenderer(TableCellRenderer renderer, int Index_row, int Index_col) {
        Component comp = super.prepareRenderer(renderer, Index_row, Index_col);

        if (green != null && green[Index_col] == Index_row && Index_col > 0) {
            comp.setBackground(Color.green);
            comp.setFont(new Font("Arial", Font.BOLD, 12));
        } else if (Index_row % 2 == 0 && !isCellSelected(Index_row, Index_col)) {
            comp.setBackground(Color.white);
        } else {
            comp.setBackground(new Color(220, 220, 220));
        }
        return comp;
    }

    // find the best segments in a split-time table
    private int[] findBestSegments(Object[][] data) {
        int[] temp = new int[size];
        double tempNumber;
        for (int i = 1; i < size; i++) {
            tempNumber = Double.MAX_VALUE;
            for (int j = 0; j < data.length; j++) {
                if ((Double) data[j][i] < tempNumber) {
                    temp[i] = j;
                    tempNumber = (Double) data[j][i];
                }
            }
        }
        return temp;
    }

    private void printDebugData() {
        int numRows = getRowCount();
        int numCols = getColumnCount();
        javax.swing.table.TableModel model = getModel();

        System.out.println("Value of data: ");
        for (int i = 0; i < numRows; i++) {
            System.out.print("    row " + i + ":");
            for (int j = 0; j < numCols; j++) {
                System.out.print("  " + model.getValueAt(i, j));
            }
            System.out.println();
        }
        System.out.println("--------------------------");
    }

    /**
     * Generates a table and colors the cells if needed.
     * @param data The data to use.
     * @param split Is the data split times or not
     */
    public void reCalc(Object[][] data, boolean split) {
        if (split == true) {
            green = findBestSegments(data);
        } else {
            green = null;
        }
        for (int i = 0; i < this.getRowCount(); i++) {
            for (int j = 0; j < this.getColumnCount(); j++) {
                setValueAt(data[i][j], i, j);
            }
        }

    }
}