/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package plugin.timeslip;

import channel.Channel;
import org.jfree.data.xy.XYDataItem;
import org.jfree.data.xy.XYSeries;

/**
 *
 * @author Per Øyvind
 */
public class DirectXYSeries extends XYSeries {

    private boolean superImposed;
    private Channel cX;
    private Channel cY;
    private int start;
    private int stop;
    private int density;
    private int disp;

    /**
     * 
     * @param key
     * @param density
     * @param cX
     * @param cY
     * @param start
     * @param stop
     * @param displacement
     * @param sImp
     */
    public DirectXYSeries(java.lang.Comparable key, int density, Channel cX, Channel cY, int start, int stop, int displacement, boolean sImp) {
        super(key);
        this.superImposed = sImp;
        this.cX = cX;
        this.cY = cY;
        this.start = start;
        this.stop = stop;
        this.density = density;
        this.disp = displacement;
        fireSeriesChanged();
    }

    /**
     * 
     * @return
     */
    public int getItemCount() {
        int correction = 0;
        if ((stop - start) % density != 0) {
            correction = 1;
        }
        return ((stop - start) / density) + correction;
    }

    /**
     * 
     * @param index
     * @return
     */
    @Override
    public XYDataItem getDataItem(int index) {
        int displacement = 0;
        if (superImposed) {
            displacement = disp;
        }
        double xValue, yValue;
        if (index == getItemCount() - 1) {
            xValue = cX.getCalibratedValues()[stop] - cX.getCalibratedValues()[displacement];
            yValue = cY.getCalibratedValues()[stop];
        } else {
            xValue = cX.getCalibratedValues()[start + (index * density)] - cX.getCalibratedValues()[displacement];
            yValue = cY.getCalibratedValues()[start + (index * density)];
        }
        return new XYDataItem(xValue, yValue);
    }

    /**
     * 
     * @param dataindex
     * @return
     */
    public double getPreciseValueX(int dataindex) {
        int displacement = 0;
        if (superImposed) {
            displacement = disp;
        }
        return cX.getCalibratedValues()[dataindex] - cX.getCalibratedValues()[displacement];
    }

    /**
     * 
     * @param dataindex
     * @return
     */
    public double getPreciseValueY(int dataindex) {
        return cY.getCalibratedValues()[dataindex];
    }

    /**
     * 
     * @param cX
     */
    public void setXAxis(Channel cX) {
        this.cX = cX;
        fireSeriesChanged();
    }

    /**
     * 
     * @param b
     */
    public void setSuperImposed(boolean b) {
        this.superImposed = b;
        fireSeriesChanged();
    }

    /**
     * 
     * @param start
     */
    public void setStart(int start) {
        this.start = start;
        fireSeriesChanged();

    }

    /**
     * 
     * @param stop
     */
    public void setStop(int stop) {
        this.stop = stop;
        fireSeriesChanged();
    }

    /**
     * 
     * @param density
     */
    public void setDensity(int density) {
        this.density = density;
        fireSeriesChanged();
    }

    /**
     * 
     * @return
     */
    public int getStart() {
        return start;
    }

    /**
     * 
     * @return
     */
    public int getStop() {
        return stop;
    }

    //Returns the largest x-value in the series, ignoring any Double.NaN values.
    /**
     * 
     * @return
     */
    public double getMaxX() {
        double tempMax = Double.NEGATIVE_INFINITY;
        for (int i = 0; i < cX.getCalibratedValues().length; i++) {
            if (cX.getCalibratedValues()[i] > tempMax) {
                tempMax = cX.getCalibratedValues()[i];
            }
        }
        return tempMax;
    }
    //Returns the largest y-value in the series, ignoring any Double.NaN values.

    /**
     * 
     * @return
     */
    public double getMaxY() {
        double tempMax = Double.NEGATIVE_INFINITY;
        for (int i = 0; i < cY.getCalibratedValues().length; i++) {
            if (cY.getCalibratedValues()[i] > tempMax) {
                tempMax = cY.getCalibratedValues()[i];
            }
        }
        return tempMax;
    }
    //Returns the smallest x-value in the series, ignoring any Double.NaN values.

    /**
     * 
     * @return
     */
    public double getMinX() {
        double tempMin = Double.POSITIVE_INFINITY;
        for (int i = 0; i < cX.getCalibratedValues().length; i++) {
            if (cX.getCalibratedValues()[i] < tempMin) {
                tempMin = cX.getCalibratedValues()[i];
            }
        }
        return tempMin;
    }
    //Returns the smallest y-value in the series, ignoring any null and Double.NaN values.

    /**
     * 
     * @return
     */
    public double getMinY() {
        double tempMin = Double.POSITIVE_INFINITY;
        for (int i = 0; i < cX.getCalibratedValues().length; i++) {
            if (cX.getCalibratedValues()[i] < tempMin) {
                tempMin = cX.getCalibratedValues()[i];
            }
        }
        return tempMin;
    }

//Returns the index of the item with the specified x-value, or a negative index if the series does not contain an item with that x-value.
    /**
     * 
     * @param x
     * @return
     */
    public int indexOf(java.lang.Number x) {
        for (int i = 0; i < getItemCount(); i++) {
            if (getDataItem(i).getX() == x) {
                return i;
            }
        }
        return -1;
    }
//Returns a new array containing the x and y values from this series.
/*public double[][] toArray(){
    return null;
    }*/
}
