/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package plugin.xy;

import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import javax.swing.JPanel;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.event.AxisChangeEvent;
import org.jfree.chart.event.AxisChangeListener;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.ui.RectangleInsets;
import segmentation.Segmentation;
import org.jfree.chart.ChartMouseEvent;
import org.jfree.chart.ChartMouseListener;
import org.jfree.chart.ChartRenderingInfo;
import org.jfree.chart.LegendItem;
import org.jfree.chart.LegendItemCollection;
import org.jfree.chart.LegendItemSource;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.ui.RectangleEdge;
import org.jfree.util.ShapeUtilities;

/**
 *
 * @author Per Øyvind
 */
public class PluginPanel extends JPanel implements AxisChangeListener, ChartMouseListener {

    private XYChartPlugin lcp;
    private ChartPanel chartPanel;
    private ArrayList<Double> xScaleSegmentationOffsets = null;
    private int drawDensity = 10;
    private boolean centerRangeAxis = false; // apply to all ranges
    private boolean centerDomainAxis = true;
    private double domainAxisMaximum;
    private double domainAxisMinimum;
    private boolean zoomLock = true;

    /**
     * 
     * @param lcp
     */
    public PluginPanel(XYChartPlugin lcp) {
        this.lcp = lcp;
       // setDoubleBuffered(true);
        setLayout(new BorderLayout());
        JFreeChart chart = createChart();
        chartPanel = new ChartPanel(chart);

        chartPanel.setMinimumDrawHeight(0);
        chartPanel.setMinimumDrawWidth(0);
        chartPanel.setMaximumDrawHeight(10000);
        chartPanel.setMaximumDrawWidth(10000);
        //chartPanel.setDoubleBuffered(true);
        chartPanel.setMouseWheelEnabled(false);
        chartPanel.setDomainZoomable(false);
        chartPanel.setRangeZoomable(false);
        
        chartPanel.addChartMouseListener(this);



        add(chartPanel, BorderLayout.CENTER);
    }

    /**
     * 
     * @param e
     */
    @Override
    public void chartMouseClicked(ChartMouseEvent e) {
        
        
        JFreeChart chart = (JFreeChart) e.getSource();
        int mouseX = e.getTrigger().getX();
        int mouseY = e.getTrigger().getY();
        Point2D p = chartPanel.translateScreenToJava2D(
                new Point(mouseX, mouseY));
        XYPlot plot = (XYPlot) chart.getPlot();
        ChartRenderingInfo info = chartPanel.getChartRenderingInfo();
        Rectangle2D dataArea = info.getPlotInfo().getDataArea();
        ValueAxis domainAxis = plot.getDomainAxis();
        RectangleEdge domainAxisEdge = plot.getDomainAxisEdge();
        double tempX = domainAxis.java2DToValue(p.getX(), dataArea,
                domainAxisEdge);
        ValueAxis rangeAxis = plot.getRangeAxis();
        RectangleEdge rangeAxisEdge = plot.getRangeAxisEdge();
        double tempY = rangeAxis.java2DToValue(p.getY(), dataArea,
                rangeAxisEdge);
        

        int offset = slowfind(tempX,
                tempY,
                lcp.getParentProject().getChannel(lcp.getXAxis()).getCalibratedValues(),
                lcp.getParentProject().getChannel(lcp.getYAxis()).getCalibratedValues());
       
        if(lcp.isCursorLinked())lcp.getParentView().setGlobalCursorPosition(offset);


    }

    /**
     * 
     * @param e
     */
    @Override
    public void chartMouseMoved(ChartMouseEvent e) {
    }

    private JFreeChart createChart() {
        JFreeChart chart = ChartFactory.createXYLineChart("", "", "",
                null, PlotOrientation.VERTICAL, true, true, false);
        XYPlot plot = chart.getXYPlot();
        plot.getDomainAxis().addChangeListener(this);
        chart.setTitle("");
        chart.setPadding(RectangleInsets.ZERO_INSETS);
        chart.setBorderVisible(false);
        chart.setAntiAlias(false);
        chart.setTextAntiAlias(false);

        chart.getRenderingHints().put(RenderingHints.KEY_STROKE_CONTROL, RenderingHints.VALUE_STROKE_PURE);
        plot.setDomainGridlinePaint(Color.GRAY);
        plot.setRangeGridlinePaint(Color.GRAY);
        plot.setDomainCrosshairPaint(Color.BLACK);
        plot.setRangeCrosshairPaint(Color.BLACK);
        plot.setDomainCrosshairStroke(new BasicStroke(1));
        plot.setRangeCrosshairStroke(new BasicStroke(1));
        plot.setBackgroundPaint(Color.WHITE);
        plot.setAxisOffset(RectangleInsets.ZERO_INSETS);
        plot.setInsets(RectangleInsets.ZERO_INSETS);
        plot.setDomainGridlinesVisible(true);
        plot.setRangeGridlinesVisible(true);
        plot.setDomainCrosshairVisible(true);
        plot.setRangeCrosshairVisible(true);
        //plot.setDomainCrosshairLockedOnData(true);
        //plot.setRangeCrosshairLockedOnData(true);

        //chartPanel.set
        return chart;
    }

    /**
     * 
     * @param ace
     */
    @Override
    public void axisChanged(AxisChangeEvent ace) {
       // maintainRanges();
        
    }

    /**
     * 
     */
    public void rebuildSeries() {
        XYPlot plot = chartPanel.getChart().getXYPlot();
          String xAxis = lcp.getXAxis();
        ArrayList<Integer> laps = lcp.getSelectedLaps();
        ArrayList<Integer> segments = lcp.getSelectedSegments();
        String yAxis = lcp.getYAxis();
        if (xAxis==null || yAxis==null || laps.isEmpty() || segments.isEmpty()) {
            plot.setDataset(null);
            return;
        }
        plot.clearRangeAxes();
        NumberAxis domainAxis = new NumberAxis();
        domainAxis.setAutoRangeIncludesZero(false);
        domainAxis.addChangeListener(this);
        plot.setDomainAxis(domainAxis);

      
        
        
      
            XYSeriesCollection dataset = new XYSeriesCollection();
            XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();
            NumberAxis axis = new NumberAxis();
            axis.setAutoRangeIncludesZero(false);
            plot.setRangeAxis(axis);
           int seriesCounter = 0;
            for (int j = 0; j < laps.size(); j++) {
                
                for (int k = 0; k < segments.size(); k++) {
                    String key = "" + xAxis + "_" + yAxis + "_" + laps.get(j) + "_" + segments.get(k);
                    if (lcp.getParentProject().getChannel(xAxis) != null
                            && lcp.getParentProject().getChannel(yAxis)!=null
                            && laps.get(j) <= lcp.getParentProject().getSegmentation().getAmountOfLaps()
                            && segments.get(k) <= (lcp.getParentProject().getSegmentation().getAmountOfSegments() + 1)) {
                        dataset.addSeries(createSeries(key, xAxis, yAxis, laps.get(j), segments.get(k)));
                    }


                        Color c = lcp.getParentProject().getLapColor((int) laps.get(j));
                      //  Color axisColor = Color.BLACK;
                    
                    renderer.setSeriesPaint(seriesCounter, c);
                    renderer.setSeriesShapesVisible(seriesCounter, true);
                    renderer.setSeriesLinesVisible(seriesCounter, false);
                     Shape cross = ShapeUtilities.createDiagonalCross(3, 1);
                    renderer.setSeriesShape(seriesCounter,  cross);
                    //renderer.setSeriesStroke(seriesCounter, new BasicStroke(2));
                   /* axis.setAxisLinePaint(axisColor);
                    axis.setLabelPaint(axisColor);
                    axis.setTickLabelPaint(axisColor);
                    axis.setTickMarkPaint(axisColor);*/
                    seriesCounter++;
                }
            }

            if (dataset.getSeriesCount() != 0) {
                plot.setDataset(dataset);
                //plot.mapDatasetToRangeAxis(i);
                plot.setRenderer(renderer);
                //plot.getRangeAxis(i)
            } else {
                plot.setDataset(null);
                plot.setRenderer(null);
            }
        

        LegendItemCollection legendItemsOld = plot.getLegendItems();
        final LegendItemCollection legendItemsNew = new LegendItemCollection();

       
            for (int i = 0; i < legendItemsOld.getItemCount(); i++) {
                LegendItem item = legendItemsOld.get(i);
                String[] key = item.getLabel().split("_");
                String newLabel = key[0] + " vs " + key[1] + " Lap "+ key[2];
                boolean exists = false;
                for (int j = 0; j < legendItemsNew.getItemCount(); j++) {
                    if (legendItemsNew.get(j).getLabel().equals(newLabel)) {
                        exists = true;
                    }
                }
                if (!exists) {
                    LegendItem newItem = new LegendItem(newLabel);
                    newItem.setFillPaint(item.getFillPaint());

                    legendItemsNew.add(newItem);
                }
            }
        

        LegendItemSource source = new LegendItemSource() {

            LegendItemCollection lic = new LegendItemCollection();

            {
                lic.addAll(legendItemsNew);
            }

            public LegendItemCollection getLegendItems() {
                return lic;
            }
        };
        LegendItemSource[] sources = new LegendItemSource[1];
        sources[0] = source;
        chartPanel.getChart().getLegend().setSources(sources);//addLegend(new LegendTitle(source));

        //updateDomainAxisMaxMin();
        //maintainRanges();
        axisChanged(null);

    }

    /**
     * 
     * @param key
     * @param xAxis
     * @param yAxis
     * @param lap
     * @param segment
     * @return
     */
    public DirectXYSeries createSeries(String key, String xAxis, String yAxis, int lap, int segment) {
        Segmentation s = lcp.getParentProject().getSegmentation();
        int start, stop;
        if (segment == 1) {
            start = s.getLapOffset(lap);
            stop = s.getSegmentOffset(lap, segment);
        } else if (segment == s.getAmountOfSegments() + 1) {
            start = s.getSegmentOffset(lap, segment - 1);
            stop = s.getLapEndOffset(lap);
        } else {
            start = s.getSegmentOffset(lap, segment - 1);
            stop = s.getSegmentOffset(lap, segment);
        }
       
        DirectXYSeries series = new DirectXYSeries(
                key,
                drawDensity,
                lcp.getParentProject().getChannel(xAxis),
                lcp.getParentProject().getChannel(yAxis),
                start,
                stop);
        return series;
    }

    /**
     * 
     * @param pos
     * @return
     */
    public boolean setCursorPosition(int pos) {
        XYPlot plot = chartPanel.getChart().getXYPlot();
        DirectXYSeries series = hasPosition(pos);
        if (series != null) {
            double value = series.getPreciseValueX(pos);
            double yValue = series.getPreciseValueY(pos);
            plot.setDomainCrosshairValue(value);
            plot.setRangeCrosshairValue(yValue);
            //maintainRanges();
            return true;
        } else {
            if(plot.getDomainCrosshairValue()!=0)
            plot.setDomainCrosshairValue(0);
        }
        return false;
    }

    /**
     * 
     * @param selected
     */
    public void setFollowDomainCursor(boolean selected) {
        this.centerDomainAxis = selected;
    }

    /**
     * 
     * @param selected
     */
    public void setFollowRangeCursor(boolean selected) {
        this.centerRangeAxis = selected;
    }
/*
    private void updateDomainAxisMaxMin() {
        double tempDomainAxisMaximum = Double.NEGATIVE_INFINITY;
        double tempDomainAxisMinimum = Double.POSITIVE_INFINITY;
        XYPlot plot = chartPanel.getChart().getXYPlot();
        for (int j = 0; j < lcp.getSelectedChannels().size(); j++) {


            XYSeriesCollection dataset = (XYSeriesCollection) plot.getDataset(j);
            if (dataset != null) {
                for (int i = 0; i < dataset.getSeriesCount(); i++) {

                    DirectXYSeries series = (DirectXYSeries) dataset.getSeries(i);
                    tempDomainAxisMaximum = Math.max(tempDomainAxisMaximum, series.getPreciseValueX(series.getStop()));
                    tempDomainAxisMinimum = Math.min(tempDomainAxisMinimum, series.getPreciseValueX(series.getStart()));
                }
            }
        }
        domainAxisMaximum = tempDomainAxisMaximum;
        domainAxisMinimum = tempDomainAxisMinimum;
    }*/

    private DirectXYSeries hasPosition(int pos) {
        XYPlot plot = chartPanel.getChart().getXYPlot();
        XYSeriesCollection dataset = (XYSeriesCollection) plot.getDataset();
        if (dataset != null) {
            for (int i = 0; i < dataset.getSeriesCount(); i++) {

                DirectXYSeries series = (DirectXYSeries) dataset.getSeries(i);
                if (pos >= series.getStart() && pos <= series.getStop()) {
                    return series;
                }
            }
        }
        return null;
    }

    /*private void maintainRanges() {
        XYPlot plot = chartPanel.getChart().getXYPlot();
        plot.getDomainAxis().removeChangeListener(this);
        double xValue = plot.getDomainCrosshairValue();
        double yValue = plot.getRangeCrosshairValue();
        double width = plot.getDomainAxis().getRange().getLength();
        double min = domainAxisMinimum;
        double max = domainAxisMaximum;
        double lower = xValue - (width / 2);
        double upper = xValue + (width / 2);
        double finalLowerX = plot.getDomainAxis().getLowerBound();
        double finalUpperX = plot.getDomainAxis().getUpperBound();
        if (finalLowerX < min) {
            finalLowerX = min;
        }
        if (finalUpperX > max) {
            finalUpperX = max;
        }

        if (centerRangeAxis) {
            plot.getRangeAxis().centerRange(yValue);
        }

        if (centerDomainAxis) {
            finalLowerX = lower;
            finalUpperX = upper;
        }
        if (zoomLock) {
            if (finalLowerX < min && finalUpperX <= max) {
                plot.getDomainAxis().setLowerBound(min);
                if (max != 0) {
                    plot.getDomainAxis().setUpperBound(Math.min(max, finalUpperX + (min - finalLowerX)));
                }
            } else if (finalLowerX >= min && finalUpperX <= max) {
                plot.getDomainAxis().setLowerBound(finalLowerX);
                plot.getDomainAxis().setUpperBound(finalUpperX);
            } else if (finalLowerX >= min && finalUpperX > max) {
                if (max != 0) {
                    plot.getDomainAxis().setUpperBound(max);
                }
                if (max != 0) {
                    plot.getDomainAxis().setLowerBound(Math.max(finalLowerX - (finalUpperX - max), min));
                }
            } else if (lower < min && upper > max) {
                if (max != 0) {
                    plot.getDomainAxis().setUpperBound(max);
                }
                plot.getDomainAxis().setLowerBound(min);
            }
        }
        plot.getDomainAxis().addChangeListener(this);
      //  updateBackGround(plot);
    }*/
/*
    private void updateBackGround(XYPlot plot) {
        if (lcp.getXAxis() == Constants.CHAN_DISTANCE) {
            int w = 500;
            int h = 1;
            Color primaryBgColor = Color.WHITE;
            Color secondaryBgColor = Color.LIGHT_GRAY;

            double lower = plot.getDomainAxis().getRange().getLowerBound();
            double upper = plot.getDomainAxis().getRange().getUpperBound();
            double width = upper - lower;
            BufferedImage image = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);
            Graphics g = image.createGraphics();
            ArrayList<Double> tempList = new ArrayList<Double>();
            int firstSegment = -1;

            tempList.add(0d);
            for (int i = 0; i < xScaleSegmentationOffsets.size(); i++) {
                double d = xScaleSegmentationOffsets.get(i);
                if (d >= lower && d <= upper) {
                    tempList.add((d - lower) / width);
                    if (firstSegment == -1) {
                        firstSegment = (i + 1);
                    }
                }
            }
            tempList.add(1d);
            if (firstSegment == -1) {
                firstSegment = 1;
                for (int i = 0; i < xScaleSegmentationOffsets.size(); i++) {
                    double d = xScaleSegmentationOffsets.get(i);
                    if (d < lower) {
                        firstSegment++;
                    } else if (d > lower) {
                        break;
                    }
                }
            }
            if (firstSegment % 2 == 0) {
                g.setColor(secondaryBgColor);
            } else {
                g.setColor(primaryBgColor);
            }
            for (int i = 0; i < tempList.size() - 1; i++) {
                double startposition = (w * tempList.get(i));
                double segwidth = (w * tempList.get(i + 1)) - startposition;
                g.drawRect((int) startposition, 0, (int) segwidth, h);
                if (g.getColor().equals(secondaryBgColor)) {
                    g.setColor(primaryBgColor);
                } else {
                    g.setColor(secondaryBgColor);
                }
            }
            plot.setBackgroundImage(image);
        } else {
            plot.setBackgroundImage(null);
        }
    }

    public void generateDomainSegmentOffsets() {
        ArrayList<Integer> offsets = lcp.getParentProject().getSegmentation().getAllSegmentOffsets();
        xScaleSegmentationOffsets = new ArrayList<Double>();
        for (Integer i : offsets) {
            xScaleSegmentationOffsets.add(lcp.getParentProject().getChannelType(Constants.CHAN_DISTANCE).getCalibratedValues()[i]);
        }
    }*/
    //Binary search
    private int slowfind(double tempX, double tempY, double[] xValues, double[] yValues) {
        if (xValues == null || xValues.length == 0 || yValues == null || yValues.length ==0) {
            return -1;
        }

       double diff = Double.POSITIVE_INFINITY; 
       int closestIndex = -1;
       for(int i = 0; i<xValues.length; i++){
           double tempDiff = Math.abs(Math.abs(tempX)-Math.abs(xValues[i])) + Math.abs(Math.abs(tempY)-Math.abs(yValues[i]));
           if(tempDiff < diff){
               if(hasPosition(i)!=null){
               
                   diff = tempDiff;
                closestIndex = i;
               }
           }
       }
      
       return closestIndex;
      }
}
