/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package segmentation;

import java.io.Serializable;
import java.util.ArrayList;

/**
 * Class to represent where a lap starts and where it ends in the data.
 * @author Lauritz Møllersen
 */
public class Lap implements Serializable {
    private int lapNumber;
    private int startOffset;
    private int endOffset;
    private ArrayList<Segment> segments = new ArrayList<Segment>();

    /**
     * Default constructor with info.
     * @param lapnumber The lap number.
     * @param startOffset The offset this lap starts at
     * @param endOffset The offset this lap ends at
     */
    public Lap(int lapnumber, int startOffset, int endOffset) {
        this.lapNumber = lapnumber;
        this.startOffset = startOffset;
        this.endOffset = endOffset;
    }

    /**
     * Default constructor without info.
     */
    public Lap() {
    }
    
    /**
     * Method to segmentate the lap in smaller parts, based on percents of the total lap distance.
     * @param distancePercents An array with percent values, in other words how long the different segments are.
     * @param values The distance values.
     */
    public void createSegments(ArrayList<Double> distancePercents, double[] values) {
        double tempPercent = 0.0;
        double tempDouble = 0.0;
        double distanceCap = 0.0;
        double startDistance = values[getStartOffset()];
        double lapDistance = values[getEndOffset()] - startDistance;
        int currentOffset = getStartOffset();
        segments = new ArrayList<Segment>();
        for(int i = 0; i < distancePercents.size(); i++) {
            tempPercent = distancePercents.get(i);
            distanceCap = startDistance + (lapDistance * tempPercent);
            while(currentOffset < getEndOffset()) {
                currentOffset++;
                if(values[currentOffset] > distanceCap) {
                    segments.add(new Segment(i+1, currentOffset));
                    break;
                }
            }
        }
    }
    
    @Override
    public String toString() {
        Segment tempSegment;
        String tempString = "Lap #" + lapNumber + " starting offset: " + getStartOffset() + " ending offset: " + getEndOffset() + "\n";
        for(int i = 0; i < segments.size(); i++) {
            tempSegment = segments.get(i);
            tempString += "\tSegment #" + tempSegment.getSegmentNumber() + " offset: " + tempSegment.getOffset() + "\n";
        }
        return tempString;
    }
    
    /**
     * @param segment
     * @return
     */
    public int getSegmentOffset(int segment) {
        Segment tempSegment = getSegment(segment);
        if(tempSegment == null) return -1;
        else return tempSegment.getOffset();
    }
    
    private Segment getSegment(int segment) {
        Segment tempSegment;
        for(int i = 0; i < segments.size(); i++) {
            tempSegment = segments.get(i);
            if(tempSegment.getSegmentNumber() == segment) {
                return tempSegment;
            }
        }
        return null;
    }
    
    /**
     * @return the lapnumber
     */
    public int getLapNumber() {
        return lapNumber;
    }

    /**
     * @param lapnumber the lapnumber to set
     */
    public void setLapNumber(int lapnumber) {
        this.lapNumber = lapnumber;
    }

    /**
     * @return the segments
     */
    public ArrayList<Segment> getSegments() {
        return segments;
    }

    /**
     * @param segments the segments to set
     */
    public void setSegments(ArrayList<Segment> segments) {
        this.segments = segments;
    }

    /**
     * @return the startOffset
     */
    public int getStartOffset() {
        return startOffset;
    }

    /**
     * @param startOffset the startOffset to set
     */
    public void setStartOffset(int startOffset) {
        this.startOffset = startOffset;
    }

    /**
     * @return the endOffset
     */
    public int getEndOffset() {
        return endOffset;
    }

    /**
     * @param endOffset the endOffset to set
     */
    public void setEndOffset(int endOffset) {
        this.endOffset = endOffset;
    }
}
