/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package segmentation;

import channel.Channel;
import java.io.Serializable;
import java.util.ArrayList;

/**
 * Class to keep track of how the channel data is split in laps and lap segments.
 * @author Lauritz Møllersen
 */
public class Segmentation implements Serializable {

    private ArrayList<Lap> laps = new ArrayList<Lap>();

    /**
     * Method to create lap objects for the lap array offsets (where does lap 2 start, where does lap 3 start, etc).
     * @param offsets The array index offsets.
     * @deprecated 
     */
    @Deprecated
    public void createLaps(ArrayList<Integer> offsets) {
        for (int i = 0; i < offsets.size() - 1; i++) {
            laps.add(new Lap(i + 1, offsets.get(i), offsets.get(i + 1)));
        }
    }

    /**
     * Method to create lap objects for the lap array offsets (where does lap 2 start, where does lap 3 start, etc).
     * @param lapChannel The lap channel values.
     */
    public void createLaps(Channel lapChannel) {
        laps = new ArrayList<Lap>();
        double[] values = lapChannel.getValues();
        int currentOffset = 0;
        int currentLap = 1;
        Lap tempLap = new Lap();
        tempLap.setLapNumber(currentLap);
        tempLap.setStartOffset(currentOffset);
        while (currentOffset < values.length) {
            if (values[currentOffset] > currentLap) {
                tempLap.setEndOffset(currentOffset);
                laps.add(tempLap);
                currentLap++;
                tempLap = new Lap();
                tempLap.setLapNumber(currentLap);
                tempLap.setStartOffset(currentOffset);
            }
            currentOffset++;
        }
        tempLap.setEndOffset(currentOffset - 1);
        laps.add(tempLap);
    }

    /**
     * Method to create lap segments for all laps.
     * @param offsets List of array index offsets, in other words where lap segment 1 starts, and so forth.
     * @param lap The lapnumber that these offsets are for.
     * @param distance The distance channel.
     * @throws Exception  
     */
    public void createSegments(ArrayList<Integer> offsets, int lap, Channel distance) throws Exception {
        Lap tempLap = getLap(lap);
        double[] values = distance.getValues();
        ArrayList<Double> distancePercents = new ArrayList<Double>();
        double lapDistance = 0.0;
        double tempDistance = 0.0;
        final int lapOffsetEnd;
        final int lapOffsetStart = tempLap.getStartOffset();
        lapOffsetEnd = tempLap.getEndOffset();

        lapDistance = values[lapOffsetEnd - 1] - values[tempLap.getStartOffset()];

        tempDistance += values[offsets.get(0)] - values[lapOffsetStart];
        distancePercents.add(tempDistance / lapDistance);
        for (int i = 0; i < offsets.size() - 1; i++) {
            tempDistance += values[offsets.get(i + 1)] - values[offsets.get(i)];
            distancePercents.add(tempDistance / lapDistance);
        }

        int nextLapOffset;
        for (int i = 0; i < laps.size(); i++) {
            nextLapOffset = getLapOffset(i + 2);
            // hack. used because there are some blank values at the end of the data array.
            if (nextLapOffset == -1) {
                nextLapOffset = values.length - 10;
            }
            laps.get(i).createSegments(distancePercents, values);

        }
        
    }

    @Override
    public String toString() {
        String tempString = "Segmentation info: \n";
        for (int i = 0; i < laps.size(); i++) {
            tempString += laps.get(i).toString();
        }
        return tempString;
    }

    /**
     * Returns the segment offsets for a given lap
     * @param lap Lap index
     * @return Lap segments
     */
    public ArrayList<Integer> getLapSegments(int lap) {
        ArrayList<Integer> offsets = new ArrayList<Integer>();
        Lap tempLap = getLap(lap);
        if (tempLap != null) {
            ArrayList<Segment> segments = tempLap.getSegments();
            for (int i = 0; i < segments.size(); i++) {
                offsets.add(segments.get(i).getOffset());
            }
        }
        return offsets;
    }

    /**
     * Returns the start lap offset for a given lap
     * @param lap Lap index
     * @return Lap start offset
     */
    public Integer getLapOffset(int lap) {
        Lap tempLap = getLap(lap);
        if (tempLap == null) {
            return -1;
        } else {
            if (lap > getAmountOfLaps()) {
                return tempLap.getStartOffset() - 1;
            } else {
                return tempLap.getStartOffset();
            }
        }
    }

    /**
     * Returns the end offset for a given lap
     * @param lap Lap index
     * @return The offset of the lap
     */
    public Integer getLapEndOffset(int lap) {
        Lap tempLap = getLap(lap);
        if (tempLap == null) {
            return -1;
        } else {
            if (lap > getAmountOfLaps()) {
                return tempLap.getEndOffset() - 1;
            } else {
                return tempLap.getEndOffset();
            }
        }
    }
    /**
     * Returns the lap object at given index
     * @param lap Lap index
     * @return Lap object
     */
    private Lap getLap(int lap) {
        Lap tempLap;
        for (int i = 0; i < laps.size(); i++) {
            tempLap = laps.get(i);
            if (tempLap.getLapNumber() == lap) {
                return tempLap;
            }
        }
        return null;

    }

    /**
     * Get an offset
     * @param lap
     * @param segment
     * @return The offset for the segment in the lap
     */
    public int getSegmentOffset(int lap, int segment) {
        Lap tempLap = getLap(lap);
        if (tempLap != null) {
            if (segment == 0) {
                return tempLap.getStartOffset();
            } else if (segment == getAmountOfSegments() + 1) {
                return tempLap.getEndOffset();
            } else {
                return tempLap.getSegmentOffset(segment);
            }
        } else {
            return -1;
        }
    }

    /**
     * 
     * @return The amount of laps
     */
    public int getAmountOfLaps() {
        return laps.size();
    }

    /**
     * 
     * @return All segments.
     */
    public ArrayList<Integer> getAllSegmentOffsets() {
        ArrayList<Integer> temp = new ArrayList<Integer>();
        for (int i = 1; i < laps.size()+1; i++) {
            Lap l = getLap(i);
            if(i!=1){
              temp.add(l.getStartOffset());
            }
            
            for (Segment s : l.getSegments()) {
           
                temp.add(s.getOffset());
            
            }
            
            if(i!=getAmountOfLaps()){
            temp.add(l.getEndOffset());
            }

        }
        return temp;
    }

    /**
     * 
     * @return The amount of segments in each lap.
     */
    public int getAmountOfSegments() {
        Lap tempLap = getLap(1);
        if (tempLap == null) {
            return -1;
        } else {
            return tempLap.getSegments().size();
        }
    }

    /**
     * 
     * @param offset The offset to check.
     * @return The lap that contains the offset.
     */
    public int getLapFromOffset(int offset) {
        Lap tempLap;
        for (int i = 0; i < laps.size(); i++) {
            tempLap = laps.get(i);
            if (tempLap.getStartOffset() <= offset && tempLap.getEndOffset() >= offset) {
                return tempLap.getLapNumber();
            }
        }
        return -1;
    }
    
    /**
     * Deletes all segments
     */
    public void deleteSegments() {
        Lap tempLap;
        for(int i = 0; i < laps.size(); i++) {
            tempLap = laps.get(i);
            tempLap.setSegments(new ArrayList<Segment>());
        }
    }
}
