 /*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package serializer;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author Per Øyvind
 */
public class Serializer {

    /**
     * Serializes an object extending the Externalizable interface to file.
     * @param o Object to serialize
     * @param file Target filepath
     * @throws IOException
     */
    public static void serializeToFile(Object o, String file) throws IOException {
        FileOutputStream baos = new FileOutputStream(file);
        ObjectOutputStream oos = new ObjectOutputStream(baos);
        oos.writeObject(o);
        oos.close();
    }

    /**
     * Serializes an object extending the Externalizable interface to file.
     * @param o Object to serialize
     * @param file Target file
     * @throws IOException
     */
    public static void serializeToFile(Object o, File file) throws IOException {
        FileOutputStream baos = new FileOutputStream(file);
        ObjectOutputStream oos = new ObjectOutputStream(baos);
        oos.writeObject(o);
        oos.close();
    }

    /**
     * Serializes an object extending the Externalizable interface to a ByteArray.
     * @param o Object to serialize
     * @return ByteArray representation of the object
     */
    public static byte[] serializeToByteArray(Object o) {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        ObjectOutputStream oos;
        try {
            oos = new ObjectOutputStream(baos);
            oos.writeObject(o);
            oos.close();
        } catch (IOException ex) {
            Logger.getLogger(Serializer.class.getName()).log(Level.SEVERE, null, ex);
            return null;
        }
        return baos.toByteArray();
    }

    /**
     * Deserializes an object extending the Externalizable interface from a file
     * @param datafile File path
     * @return Object
     * @throws IOException
     * @throws ClassNotFoundException
     */
    public static Object deserializeFromFile(String datafile) throws IOException, ClassNotFoundException {
        FileInputStream bais = new FileInputStream(datafile);
        ObjectInputStream ois = new ObjectInputStream(bais);
        Object o = ois.readObject();
        ois.close();
        return o;
    }

    /**
     * Deserializes an object extending the Externalizable interface from a file
     * @param datafile Input File
     * @return Object
     * @throws IOException
     * @throws ClassNotFoundException
     */
    public static Object deserializeFromFile(File datafile) throws IOException, ClassNotFoundException {
        FileInputStream bais = new FileInputStream(datafile);
        ObjectInputStream ois = new ObjectInputStream(bais);
        Object o = ois.readObject();
        ois.close();
        return o;
    }

    /**
     * Deserializes an object extending the Externalizable interface from a ByteArray.
     * @param data Input ByteArray
     * @return Object
     * @throws IOException
     * @throws ClassNotFoundException
     */
    public static Object deserializeFromByteArray(byte[] data) throws IOException, ClassNotFoundException {
        ByteArrayInputStream bais = new ByteArrayInputStream(data);
        ObjectInputStream ois;
        ois = new ObjectInputStream(bais);
        Object o;
        o = ois.readObject();
        ois.close();
        return o;
    }
}