/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package view;

import java.awt.Component;
import java.util.ArrayList;
import project.Project;
import plugin.Plugin;

/**
 * The View interface.
 * @author Per Øyvind Stadheim
 */
public interface View {

    /**
     * Initializes the view by performing necessary operations
     * and setting the parent project reference. This method needs 
     * to check if the view's fields are already set (i.e. the view
     * is loaded from disk).
     * @param p The parent project of the view.
     * @throws ViewInitException Exception used to abort the initialization
     * of the view
     */
    public void init(Project p) throws ViewInitException;

    /**
     * Sets the title of the view.
     * This method is only used by the ViewFactory for initialization purposes.
     * @param title
     */
    public void setTitle(String title);

    /**
     * Get the parent project of the view.
     * @return The parent project.
     */
    public Project getParentProject();

    /**
     * Sets the title of the view
     * @param title The view title
     * @param undo
     */
    public void setTitle(String title, boolean undo);

    /**
     * Returns the current title of the view
     * @return The view title
     */
    public String getTitle();

    /**
     * Method use to set the view's cursor position and update
     * any linked plugins.
     * @param lineNumber The position to set the cursor to
     */
    public void setGlobalCursorPosition(int lineNumber);

    /**
     * Returns the view's cursor position.
     * @return Cursor position
     */
    public int getGlobalCursorPosition();

    /**
     * Registers a plugin to the view.
     * @param p Plugin to register.
     * @param undo  
     */
    public void registerPlugin(Plugin p, boolean undo);

    /**
     * Removes a plugin from the view.
     * @param p Plugin to remove.
     * @param undo  
     */
    public void removePlugin(Plugin p, boolean undo);

    /**
     * Returns the list of plugins contained in the view
     * @return List of plugins
     */
    public ArrayList<Plugin> getPlugins();

    /**
     * Method used to update the plugin titles in the view
     */
    public void pluginTitlesChanged();

    /**
     * Get the main GUI component of the View.
     * @return GUI component.
     */
    public Component getViewPanel();

    /**
     * Get the property GUI component of the View.
     * @return GUI component.
     */
    public Component getPropertiesPanel();
}
