package view;

import java.io.File;
import java.io.IOException;
import serializer.Serializer;

/**
 * Class used to instantiate View objects by Java reflection.
 * @author Per Øyvind Stadheim
 */
public class ViewFactory {

    /**
     * Creates a new view instance, based on classpath.
     * @param type The classpath of the view to be instantiated.
     * @param viewCounter 
     * @return The new view instance.
     */
    public static View createNewView(Class type, int viewCounter) {
        try {
            View newView = (View) type.newInstance();
            newView.setTitle("View " + viewCounter);
            return newView;
        } catch (InstantiationException ex) {
        } catch (IllegalAccessException ex) {
        }
        return null;
    }

    /**
     * Serializes a view to a ByteArray
     * @param v View to serialize
     * @return ByteArray represention of the view
     */
    public static byte[] serializeToByteArray(View v) {
        return Serializer.serializeToByteArray(v);
    }

    /**
     * Deserializes a view from a ByteArray
     * @param b Input ByteArray
     * @return View
     * @throws IOException
     * @throws ClassNotFoundException
     */
    public static View deserializeFromByteArray(byte[] b) throws IOException, ClassNotFoundException {
        View v = (View) Serializer.deserializeFromByteArray(b);
        return v;
    }

    /**
     * Serializes a view to file
     * @param v View to serialize
     * @param file Target file
     * @throws IOException
     */
    public static void serializeToFile(View v, File file) throws IOException {
        Serializer.serializeToFile(v, file);
    }

    /**
     * Deserializes a view from file
     * @param file Input file
     * @return View
     * @throws IOException
     * @throws ClassNotFoundException
     */
    public static View deserializeFromFile(File file) throws IOException, ClassNotFoundException {
        View v = (View) Serializer.deserializeFromFile(file);
        return v;
    }
}