/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package view.basic;

import java.awt.Component;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.util.Arrays;
import java.util.HashMap;
import javax.swing.JPanel;
import plugin.Plugin;
import project.Command;
import project.Project;
import view.DefaultView;
import view.ViewInitException;

/**
 *
 * @author Per Øyvind
 */
public class BasicView extends DefaultView {

    private BasicViewPanel viewPanel;
    private HashMap layoutValues;

    /**
     * Returns a HashMap containing the layout values for this view.
     * @return The HashMap containing the layout values
     */
    public HashMap getLayoutValues() {
        return layoutValues;
    }

    @Override
    public void init(Project p) throws ViewInitException {
        super.init(p);
        if (layoutValues == null) {
            layoutValues = new HashMap();
        }
        viewPanel = new BasicViewPanel(this);
        for (Plugin plugin : getPlugins()) {

            viewPanel.registerPlugin(plugin);
        }


    }

    @Override
    public void registerPlugin(Plugin p, boolean undo) {
        super.registerPlugin(p, undo);
        if (getPlugins().contains(p) && !undo) {
            this.viewPanel.registerPlugin(p);
        }
        viewPanel.layoutChanged();
    }

    @Override
    public void removePlugin(Plugin p, boolean undo) {
        super.removePlugin(p, undo);
        if (!getPlugins().contains(p)) {
            viewPanel.removePlugin(p);
        }
    }

    /**
     * Method used to tell the view to update the plugin titles.
     */
    @Override
    public void pluginTitlesChanged() {
        viewPanel.refreshPluginTitles();
    }

    /**
     * Sets the layout value for a specific plugin within the view.
     * @param plugin The target plugin
     * @param values The layout values
     * @param undo 
     */
    public void setLayoutValue(Plugin plugin, double[] values, boolean undo) {
        if (!undo) {
            if (!Arrays.equals(values, ((double[]) layoutValues.get(plugin)))) {
                layoutValues.remove(plugin);
                layoutValues.put(plugin, values);
                viewPanel.layoutChanged();
            }
        } else {
            final Plugin targetPlugin = plugin;
            final double[] oldValue = (double[]) layoutValues.get(plugin);
            final double[] newValue = values;
            final BasicView source = this;
            Command c = new Command() {

                @Override
                public void execute() {
                    source.setLayoutValue(targetPlugin, newValue, false);
                }

                @Override
                public void reverse() {
                    source.setLayoutValue(targetPlugin, oldValue, false);
                }

                @Override
                public String toString() {
                    return "Change Layout";
                }
            };
            c.execute();
            getParentProject().addUndoHistory(c);

        }
    }

    @Override
    public Component getViewPanel() {
        return this.viewPanel;
    }

    @Override
    public Component getPropertiesPanel() {
        return new JPanel();
    }

    @Override
    public void writeExternal(ObjectOutput out) throws IOException {
        super.writeExternal(out);
        out.writeObject(indexBasedLayoutValues(layoutValues));
    }

    @Override
    public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
        super.readExternal(in);
        layoutValues = referenceBasedLayoutValues((HashMap) in.readObject());
    }

    private HashMap indexBasedLayoutValues(HashMap values) {
        HashMap temp = new HashMap();
        for (int i = 0; i < getPlugins().size(); i++) {
            temp.put(i, values.get(getPlugins().get(i)));



        }
        return temp;
    }

    private HashMap referenceBasedLayoutValues(HashMap values) {
        HashMap temp = new HashMap();
        for (int i = 0; i < getPlugins().size(); i++) {
            temp.put(getPlugins().get(i), values.get(i));
        }
        return temp;
    }
}
