/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * BasicViewPanel.java
 *
 * Created on Apr 18, 2012, 11:45:08 AM
 */
package view.basic;

import java.awt.Rectangle;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.image.BufferedImage;
import java.beans.PropertyVetoException;
import java.io.IOException;
import java.net.URL;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JDesktopPane;
import javax.swing.JInternalFrame;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;
import plugin.Plugin;

/**
 *
 * @author Per Øyvind
 */
public class BasicViewPanel extends javax.swing.JPanel implements InternalFrameListener, ComponentListener {

    private final BasicView bv;
    private final boolean snap;

    /** Creates new form BasicViewPanel
     * @param bv The owner view
     */
    public BasicViewPanel(BasicView bv) {
        initComponents();
        this.bv = bv;
        this.snap = true;
        BufferedImage image = null;
        try {
            image = ImageIO.read(getClass().getClassLoader().getResource("view/basic/logo.png"));
        } catch (IOException ex) {
            Logger.getLogger(BasicViewPanel.class.getName()).log(Level.SEVERE, null, ex);
        }
        customJDesktopPane1.setImage(image);
        customJDesktopPane1.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);
        customJDesktopPane1.addComponentListener(this);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        customJDesktopPane1 = new view.basic.CustomJDesktopPane();

        customJDesktopPane1.setBackground(new java.awt.Color(204, 204, 204));

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(customJDesktopPane1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 680, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(customJDesktopPane1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 444, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private view.basic.CustomJDesktopPane customJDesktopPane1;
    // End of variables declaration//GEN-END:variables

    /**
     * Adds a plugin to the view panel.
     * @param p Plugin to register
     */
    public void registerPlugin(Plugin p) {
        PluginFrame frame = new PluginFrame(p);
        ImageIcon icon;
        URL url = (getClass().getClassLoader().getResource("gui/icon.png"));
        icon = new ImageIcon(url);
        if (icon != null) {
            frame.setFrameIcon(icon);
        }
        frame.pack();
        frame.addInternalFrameListener(this);
        frame.setVisible(true);
        customJDesktopPane1.add(frame);
        frame.requestFocusInWindow();
        try {
            frame.setSelected(true);
        } catch (PropertyVetoException ex) {
            Logger.getLogger(BasicViewPanel.class.getName()).log(Level.SEVERE, null, ex);
        }
        frame.toFront();
        frame.addComponentListener(this);

        double[] values = getRelativeBounds(frame.getBounds());
        if (bv.getLayoutValues().get(frame.getPlugin()) == null) {
            bv.getLayoutValues().put(frame.getPlugin(), values);

        }
    }

    /**
     * Removes a plugin from the view panel.
     * @param p Plugin to remove
     */
    public void removePlugin(Plugin p) {
        for (JInternalFrame frame : customJDesktopPane1.getAllFrames()) {
            if (((PluginFrame) frame).getPlugin().equals(p)) {
                customJDesktopPane1.remove(frame);
                customJDesktopPane1.repaint();

            }
        }
    }

    /**
     * Method used to update the plugin layout.
     */
    public void layoutChanged() {
        double desktopWidth = bv.getParentProject().getGui().getViewAreaDimension().width;//bv.getParentProject().getGui().getViewAreaDimension().width;//customJDesktopPane1.getWidth();
        double desktopHeight = bv.getParentProject().getGui().getViewAreaDimension().height;  //customJDesktopPane1.getVisibleRect().height;//bv.getParentProject().getGui().getViewAreaDimension().height;//customJDesktopPane1.getHeight();
        for (JInternalFrame frame : customJDesktopPane1.getAllFrames()) {
            PluginFrame pluginFrame = (PluginFrame) frame;
            double[] values = (double[]) bv.getLayoutValues().get(pluginFrame.getPlugin());
            int pluginX = (int) Math.round(values[0] * desktopWidth);
            int pluginY = (int) Math.round(values[1] * desktopHeight);
            int pluginWidth = (int) Math.round(values[2] * desktopWidth);
            int pluginHeight = (int) Math.round(values[3] * desktopHeight);
            if (snap) {
                pluginX = findClosestGridPoint(pluginX);
                pluginY = findClosestGridPoint(pluginY);
                pluginWidth = findClosestGridPoint(pluginWidth);
                pluginHeight = findClosestGridPoint(pluginHeight);
            }
            pluginFrame.removeComponentListener(this);
            pluginFrame.setBounds(pluginX, pluginY, pluginWidth, pluginHeight);
            pluginFrame.addComponentListener(this);
        }
        customJDesktopPane1.repaint();
    }

    /**
     * Method used to refresh all plugin titles in the view panel.
     */
    public void refreshPluginTitles() {
        for (JInternalFrame frame : customJDesktopPane1.getAllFrames()) {
            PluginFrame pluginFrame = (PluginFrame) frame;
            pluginFrame.setTitle(pluginFrame.getPlugin().getTitle());
        }
    }

    @Override
    public void internalFrameOpened(InternalFrameEvent e) {
    }

    @Override
    public void internalFrameClosing(InternalFrameEvent e) {
    }

    @Override
    public void internalFrameClosed(InternalFrameEvent e) {
        bv.removePlugin(((PluginFrame) e.getSource()).getPlugin(), true);
    }

    @Override
    public void internalFrameIconified(InternalFrameEvent e) {
    }

    @Override
    public void internalFrameDeiconified(InternalFrameEvent e) {
    }

    @Override
    public void internalFrameActivated(InternalFrameEvent e) {
        bv.getParentProject().getGui().setCurrentPlugin(((PluginFrame) e.getSource()).getPlugin());
    }

    @Override
    public void internalFrameDeactivated(InternalFrameEvent e) {
    }

    @Override
    public void componentResized(ComponentEvent e) {
        if (e.getSource() instanceof PluginFrame) {
            PluginFrame frame = (PluginFrame) e.getSource();
            Rectangle r;
            double[] values = null;
            if (snap) {
                int newW = findClosestGridPoint(frame.getX() + frame.getWidth());
                int newH = findClosestGridPoint(frame.getY() + frame.getHeight());
                if (newW != frame.getX() + frame.getWidth() || newH != frame.getHeight()) {
                    r = new Rectangle(frame.getX(), frame.getY(), newW - frame.getX(), newH - frame.getY());
                    values = getRelativeBounds(r.getBounds());
                }
            } else {

                values = getRelativeBounds(frame.getBounds());
            }
            if (values != null) {
                bv.setLayoutValue(frame.getPlugin(), values, true);
            }
        } else if (e.getSource() instanceof CustomJDesktopPane) {
            layoutChanged();
        }
    }

    @Override
    public void componentMoved(ComponentEvent e) {
        if (e.getSource() instanceof PluginFrame) {
            PluginFrame frame = (PluginFrame) e.getSource();
            Rectangle r;
            double[] values = null;
            if (snap) {
                int newX = findClosestGridPoint(frame.getX());
                int newY = findClosestGridPoint(frame.getY());
                if (newX != frame.getX() || newY != frame.getY()) {
                    r = new Rectangle(newX, newY, frame.getWidth(), frame.getHeight());
                    values = getRelativeBounds(r.getBounds());
                }
            } else {
                values = getRelativeBounds(frame.getBounds());
            }
            if (values != null) {
                bv.setLayoutValue(frame.getPlugin(), values, true);
            }
        }
    }

    @Override
    public void componentShown(ComponentEvent e) {
    }

    @Override
    public void componentHidden(ComponentEvent e) {
    }

    private double[] getRelativeBounds(Rectangle frame) {
        double values[] = new double[4];
        int viewWidth = bv.getParentProject().getGui().getViewAreaDimension().width;//customJDesktopPane1.getWidth();
        int viewHeight = bv.getParentProject().getGui().getViewAreaDimension().height;//customJDesktopPane1.getHeight();
        values[0] = frame.getX() / viewWidth;
        values[1] = frame.getY() / viewHeight;
        values[2] = frame.getWidth() / viewWidth;
        values[3] = frame.getHeight() / viewHeight;
        return values;
    }

    private int findClosestGridPoint(int x) {
        int gridSize = (int) Math.round(bv.getParentProject().getGui().getViewAreaDimension().width / 50);
        int lower = x;
        int upper = x;
        while (lower % gridSize != 0) {
            lower--;
        }
        while (upper % gridSize != 0) {
            upper++;
        }
        if (upper - x < x - lower) {
            return upper;
        } else {
            return lower;
        }
    }
}
